# -*- coding: utf8 -*-

"""
clikraken.clikraken_cmd

This module handles the parsing of the command line arguments
and associates the different subcommands with the corresponding
function to be called.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

import argparse
import codecs
import textwrap
from decimal import Decimal
import sys

import clikraken.global_vars as gv
import clikraken.clikraken_utils as ck_utils

from clikraken.api.public.asset_pairs import asset_pairs
from clikraken.api.public.depth import depth
from clikraken.api.public.last_trades import last_trades
from clikraken.api.public.ticker import ticker
from clikraken.api.public.ohlc import ohlc

from clikraken.api.private.cancel_order import cancel_order
from clikraken.api.private.get_balance import get_balance
from clikraken.api.private.get_deposit_addresses import get_deposit_addresses
from clikraken.api.private.get_deposit_methods import get_deposit_methods
from clikraken.api.private.get_ledgers import get_ledgers
from clikraken.api.private.get_trade_balance import get_trade_balance
from clikraken.api.private.list_closed_orders import list_closed_orders
from clikraken.api.private.list_open_orders import list_open_orders
from clikraken.api.private.list_open_positions import list_open_positions
from clikraken.api.private.place_order import place_order
from clikraken.api.private.trades import trades


def parse_args():
    """
    Argument parsing

    The client works by giving general options, a subcommand
    and then options specific to the subcommand.

    For example:

        clikraken ticker                    # just a subcommand
        clikraken depth --pair XETHZEUR     # subcommand option
        clikraken ohlc -i 15 -s 1508513700
        clikraken --raw olist               # global option
        clikraken place buy 0.1337 10.42    # subcommand argument
    """

    # some help strings that are repeated many times
    pairs_help = "comma delimited list of asset pairs"
    pair_help = "asset pair"

    epilog_str = textwrap.dedent("""\
        To get help about a subcommand use: clikraken SUBCOMMAND --help
        For example:
            clikraken place --help

        Current default currency pair: {dp}.

        Create or edit the setting file {usp} to change it.
        If the setting file doesn't exist yet, you can create one by doing:
            clikraken generate_settings > {usp}

        You can also set the CLIKRAKEN_DEFAULT_PAIR environment variable
        which has precedence over the settings from the settings file.
        """.format(dp=gv.DEFAULT_PAIR, usp=gv.USER_SETTINGS_PATH))

    parser = argparse.ArgumentParser(
        description='clikraken - Command line client for the Kraken exchange',
        epilog=epilog_str,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-V', '--version', action='store_const', const=ck_utils.version, dest='main_func',
                        help='show program version')
    parser.add_argument('--debug', action='store_true', help='debug mode')
    parser.add_argument('--raw', action='store_true', help='output raw json results from the API')
    parser.add_argument('--csv', action='store_true', help='output results from the API as CSV')
    parser.add_argument('--csvseparator', default=';', help='separator character to use with CSV output')
    parser.add_argument('--cron', action='store_true',
                        help='activate cron mode (tone down errors due to timeouts or unavailable Kraken service)')
    parser.set_defaults(main_func=None)

    subparsers = parser.add_subparsers(dest='subparser_name', help='available subcommands')

    # Generate setting.ini
    parser_gen_settings = subparsers.add_parser(
        'generate_settings',
        help='[clikraken] Print default settings.ini to stdout',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_gen_settings.set_defaults(sub_func=ck_utils.output_default_settings_ini)

    # ----------
    # Public API
    # ----------

    # Asset Pairs
    parser_asset_pairs = subparsers.add_parser(
        'asset_pairs',
        aliases=['ap'],
        help='[public] Get the list of available asset pairs',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_asset_pairs.set_defaults(sub_func=asset_pairs)

    # Ticker
    parser_ticker = subparsers.add_parser(
        'ticker',
        aliases=['t'],
        help='[public] Get the ticker',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_ticker.add_argument('-p', '--pair', default=gv.TICKER_PAIRS,
                               help=pairs_help + " to get info on. ")
    parser_ticker.set_defaults(sub_func=ticker)

    # Market depth (Order book)
    parser_depth = subparsers.add_parser(
        'depth',
        aliases=['d'],
        help='[public] Get the current market depth data',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_depth.add_argument('-p', '--pair', default=gv.DEFAULT_PAIR, help=pair_help)
    parser_depth.add_argument('-c', '--count', type=int, default=7, help="maximum number of asks/bids.")
    parser_depth.set_defaults(sub_func=depth)

    # List of last trades
    parser_last_trades = subparsers.add_parser(
        'last_trades',
        aliases=['lt'],
        help='[public] Get the last trades',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_last_trades.add_argument('-p', '--pair', default=gv.DEFAULT_PAIR, help=pair_help)
    parser_last_trades.add_argument('-s', '--since', default=None,
                                    help="return trade data since given id")
    parser_last_trades.add_argument('-c', '--count', type=int, default=15, help="maximum number of trades.")
    parser_last_trades.set_defaults(sub_func=last_trades)

    # Open High Low Close data
    parser_ohlc = subparsers.add_parser(
        'ohlc',
        aliases=['oh'],
        help='[public] Get the ohlc data',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_ohlc.add_argument(
        '-p', '--pair', default=gv.DEFAULT_PAIR, help=pair_help)
    parser_ohlc.add_argument(
        '-i', '--interval', default=1,
        help="return ohlc data for interval in minutes; 1, 5, 15, 30, 60, 240, 1440, 10800, 21600.")
    parser_ohlc.add_argument('-s', '--since', default=None,
                             help="return ohlc data since given id")
    parser_ohlc.add_argument('-c', '--count', type=int,
                             default=50, help="maximum number of intervals.")
    parser_ohlc.set_defaults(sub_func=ohlc)

    # -----------
    # Private API
    # -----------

    # User balance
    parser_balance = subparsers.add_parser(
        'balance',
        aliases=['bal'],
        help='[private] Get your current balance',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_balance.set_defaults(sub_func=get_balance)

    # User trade balance
    parser_trade_balance = subparsers.add_parser(
        'trade_balance',
        aliases=['tbal'],
        help='[private] Get your current trade balance',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_trade_balance.set_defaults(sub_func=get_trade_balance)

    # Place an order
    parser_place = subparsers.add_parser(
        'place',
        aliases=['p'],
        help='[private] Place an order',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_place.add_argument('type', choices=['sell', 'buy'])
    parser_place.add_argument('volume', type=Decimal)
    parser_place.add_argument('price', default=None, nargs='?')
    parser_place.add_argument('-l', '--leverage', default="none", help='leverage for margin trading')
    parser_place.add_argument('-p', '--pair', default=gv.DEFAULT_PAIR, help=pair_help)
    parser_place.add_argument('-t', '--ordertype', choices=['market', 'limit'], default='limit',
                              help="order type. Currently implemented: [limit, market].")
    parser_place.add_argument('-s', '--starttm', default=0, help="scheduled start time")
    parser_place.add_argument('-e', '--expiretm', default=0, help="expiration time")
    parser_place.add_argument('-r', '--userref', help="user reference id.  32-bit signed number.  (optional)")
    parser_place.add_argument('-q', '--viqc', action='store_true', help="volume in quote currency")
    parser_place.add_argument('-T', '--nopost', action='store_true',
                              help="disable 'post-only' option (for limit taker orders)")
    parser_place.add_argument('-v', '--validate', action='store_true', help="validate inputs only. do not submit order")
    parser_place.set_defaults(sub_func=place_order)

    # cancel an order
    parser_cancel = subparsers.add_parser(
        'cancel',
        aliases=['x'],
        help='[private] Cancel orders',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_cancel.add_argument('order_ids', type=str, nargs='+', help="transaction ids")
    parser_cancel.set_defaults(sub_func=cancel_order)

    # List of open orders
    parser_olist = subparsers.add_parser(
        'olist',
        aliases=['ol'],
        help='[private] Get a list of your open orders',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_olist.add_argument('-p', '--pair', default=None, help=pair_help)
    parser_olist.add_argument('-i', '--txid', default=None,
                              help='comma delimited list of transaction ids to query info about (20 maximum)')
    parser_olist.set_defaults(sub_func=list_open_orders)

    # List of open positions
    parser_oplist = subparsers.add_parser(
        'positions',
        aliases=['pos'],
        help='[private] Get a list of your open positions',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_oplist.set_defaults(sub_func=list_open_positions)

    # List of closed orders
    parser_clist = subparsers.add_parser(
        'clist',
        aliases=['cl'],
        help='[private] Get a list of your closed orders',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_clist.add_argument('-p', '--pair', default=None, help=pair_help)
    parser_clist.add_argument('-i', '--txid', default=None,
                              help='comma delimited list of transaction ids to query info about (20 maximum)')
    parser_clist.set_defaults(sub_func=list_closed_orders)

    # Get ledgers info
    parser_ledgers = subparsers.add_parser(
        'ledgers',
        aliases=['lg'],
        help='[private] Get ledgers info',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_ledgers.add_argument(
        '-a', '--asset',
        default='all',
        help='comma delimited list of assets to restrict output to')
    parser_ledgers.add_argument(
        '-t', '--type',
        default='all',
        help='type of ledger to retrieve. Possible values: all|deposit|withdrawal|trade|margin')
    parser_ledgers.add_argument(
        '-s', '--start',
        default=None,
        help='starting unix timestamp or ledger id of results (exclusive)')
    parser_ledgers.add_argument(
        '-e', '--end',
        default=None,
        help='ending unix timestamp or ledger id of results (exclusive)')
    parser_ledgers.add_argument(
        '-o', '--ofs',
        default=None,
        help='result offset')
    parser_ledgers.add_argument(
        '-i', '--id',
        default=None,
        help='comma delimited list of ledger ids to query info about (20 maximum)')
    parser_ledgers.set_defaults(sub_func=get_ledgers)

    # Get trades info
    parser_trades = subparsers.add_parser(
        'trades',
        aliases=['tr'],
        help='[private] Get trades history',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_trades.add_argument(
        '-t', '--type',
        default='all',
        help='type of trade. Values: all|any position|closed position|closing position|no position')
    # TODO: trades parameter
    parser_trades.add_argument(
        '-s', '--start',
        default=None,
        help='starting unix timestamp or trade tx id of results (exclusive)')
    parser_trades.add_argument(
        '-e', '--end',
        default=None,
        help='ending unix timestamp or trade tx id of results (exclusive)')
    parser_trades.add_argument(
        '-o', '--ofs',
        default=None,
        help='result offset')
    parser_trades.add_argument(
        '-i', '--id',
        default=None,
        help='comma delimited list of transaction ids to query info about (20 maximum)')
    parser_trades.add_argument('-p', '--pair', default=None, help=pair_help)
    parser_trades.set_defaults(sub_func=trades)

    # User Funding

    # Deposit Methods
    parser_deposit_methods = subparsers.add_parser(
        'deposit_methods',
        aliases=['dm'],
        help='[private] Get deposit methods',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_deposit_methods.add_argument('-a', '--asset', default=gv.DEFAULT_ASSET, help='asset being deposited')
    parser_deposit_methods.set_defaults(sub_func=get_deposit_methods)

    # Deposit Addresses
    parser_deposit_addresses = subparsers.add_parser(
        'deposit_addresses',
        aliases=['da'],
        help='[private] Get deposit addresses',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser_deposit_addresses.add_argument('-a', '--asset', default=gv.DEFAULT_ASSET, help='asset being deposited')
    parser_deposit_addresses.add_argument('-m', '--method', default=None, help='name of the deposit method')
    parser_deposit_addresses.add_argument('-n', '--new', action='store_true',
                                          help="whether or not to generate a new address")
    parser_deposit_addresses.add_argument('-1', '--one', action='store_true', help="return just one address")
    parser_deposit_addresses.set_defaults(sub_func=get_deposit_addresses)

    args = parser.parse_args()

    # make sure that either sub_func or main_func is defined
    # otherwise just print usage and exit
    # (this weird construction is a hack to work around Python bug #9351 https://bugs.python.org/issue9351)
    if all([vars(args).get(f, None) is None
            for f in ['sub_func', 'main_func']]):
        parser.print_usage()
        sys.exit(0)

    gv.CRON = args.cron

    # Trick from https://stackoverflow.com/a/37059682/862188
    # in order to be able to parse things like "\t" or "\\" for example
    separator = codecs.escape_decode(bytes(args.csvseparator, "utf-8"))[0].decode("utf-8")
    gv.CSV_SEPARATOR = separator

    return args



# -*- coding: utf8 -*-

"""
clikraken.api.private.get_trade_balance

This module queries the TradeBalance method of Kraken's API
and outputs the results.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import _tabulate as tabulate


def get_trade_balance(args=None):
    """Get user trade balance."""

    # Parameters to pass to the API
    api_params = {}

    res = query_api('private', 'TradeBalance', api_params, args)

    tbal_list = [
        ['equivalent balance', res.get('eb', 'n/a')],
        ['trade balance', res.get('tb', 'n/a')],
        ['margin amount of open positions', res.get('m', 'n/a')],
        ['cost basis of open positions', res.get('c', 'n/a')],
        ['current floating valuation of open positions', res.get('v', 'n/a')],
        ['equity', res.get('e', 'n/a')],
        ['free margin', res.get('mf', 'n/a')],
        ['margin level', res.get('ml', 'n/a')],
        ['unrealized net profit/loss of open positions', res.get('n', 'n/a')],
    ]
    print(tabulate(tbal_list))



from clikraken import clikraken_utils


def test_asset_pair_short():
    assert 'ETHEUR' == clikraken_utils.asset_pair_short('XETHZEUR')
    assert 'ETHEUR' == clikraken_utils.asset_pair_short('ETHEUR')
    assert 'DASHEUR' == clikraken_utils.asset_pair_short('DASHEUR')
    assert 'DASHXBT' == clikraken_utils.asset_pair_short('DASHXBT')


def test_base_quote_short_from_asset_pair():
    assert 'ETH', 'EUR' == clikraken_utils.base_quote_short_from_asset_pair('XETHZEUR')
    assert 'BCH', 'USD' == clikraken_utils.base_quote_short_from_asset_pair('BCHUSD')
    assert 'DASH', 'EUR' == clikraken_utils.base_quote_short_from_asset_pair('DASHEUR')
    assert 'DASH', 'XBT' == clikraken_utils.base_quote_short_from_asset_pair('DASHXBT')



# -*- coding: utf8 -*-

"""
clikraken.api.private.list_list_closed_orders

This module queries the ClosedOrders method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from decimal import Decimal

from clikraken.api.api_utils import parse_order_res, query_api
from clikraken.clikraken_utils import asset_pair_short
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv


def list_closed_orders(args):
    """List closed orders."""

    # Parameters to pass to the API
    api_params = {
        # TODO
    }
    if args.txid:
        api_params.update({'txid': args.txid})
        res_ol = query_api('private', 'QueryOrders', api_params, args)
    else:
        res = query_api('private', 'ClosedOrders', api_params, args)
        # extract list of orders from API results
        res_ol = res['closed']

    # the parsing is done in an helper function
    ol = parse_order_res(res_ol, ['closed', 'canceled'])

    # merge order types in one list
    ol = ol['buy'] + ol['sell']

    # filter out orders with zero volume executed
    ol = [odict for odict in ol if Decimal(odict['vol_exec']) > 0]
    if 'pair' in args and args.pair:
        ol = [odict for odict in ol if odict['pair'] in [args.pair, asset_pair_short(args.pair)]]

    if not ol:
        return

    # sort by date
    ol = sorted(ol, key=lambda odict: odict['closing_date'])

    if args.csv:
        print(csv(ol, headers="keys"))
    else:
        print(tabulate(ol, headers="keys"))



# -*- coding: utf8 -*-

"""
clikraken.global_vars

This module serves as a container for global variables.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

import os

# Resolve userpath to an absolute path
DEFAULT_KRAKEN_API_KEYFILE = os.path.expanduser('~/.config/clikraken/kraken.key')
# If the environment variable is set, override the default value
KRAKEN_API_KEYFILE = os.getenv('CLIKRAKEN_API_KEYFILE', DEFAULT_KRAKEN_API_KEYFILE)
KRAKEN_API_KEYFILE = os.path.normpath(KRAKEN_API_KEYFILE)

# Resolve userpath to an absolute path
DEFAULT_USER_SETTINGS_PATH = os.path.expanduser('~/.config/clikraken/settings.ini')
# If the environment variable is set, override the default value
USER_SETTINGS_PATH = os.getenv('CLIKRAKEN_USER_SETTINGS_PATH', DEFAULT_USER_SETTINGS_PATH)
USER_SETTINGS_PATH = os.path.normpath(USER_SETTINGS_PATH)

# Default settings
DEFAULT_SETTINGS_INI = """[clikraken]
# default currency pair when no option '-p' or '--pair' is given
# and the environment variable CLIKRAKEN_DEFAULT_PAIR is not set
currency_pair = XETHZEUR

# default asset when no option '-a' or '--asset' is given
# and the environment variable CLIKRAKEN_DEFAULT_ASSET is not set
asset = ETH

# default currency pair list for the ticker command when no
# option '-p' or '--pair' is given and the environment variable
# CLIKRAKEN_TICKER_PAIRS is not set
ticker_currency_pairs = XBTUSD,XBTEUR,ETHUSD,ETHEUR

# Timezone for displaying date and time infos
timezone = Europe/Berlin

# API Trading Agreement
# (change to "agree" after reading https://www.kraken.com/u/settings/api)
trading_agreement = not_agree
"""

# Global variables to be shared between clikraken's modules
KRAKEN_API = None
DEFAULT_PAIR = None
TICKER_PAIRS = None
DEFAULT_ASSET = None
TZ = None
TRADING_AGREEMENT = None
CRON = None
API_KEY_LOADED = None
CSV_SEPARATOR = None



#!/usr/bin/env python3
# -*- coding: utf8 -*-

"""
clikraken

Command line client for the Kraken exchange

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from clikraken.api.api_utils import load_api_keyfile
from clikraken.clikraken_cmd import parse_args
from clikraken.clikraken_utils import load_config


def main():
    """Entrypoint for clikraken"""

    load_config()
    load_api_keyfile()

    # parse arguments
    args = parse_args()

    # args.sub_func contains the function to be called
    # depending on the chosen subcommand. If no subcomand
    # is called, then use the function given by args.main_func
    func = args.sub_func if 'sub_func' in args else args.main_func

    if callable(func):
        func(args)



# -*- coding: utf8 -*-

"""
clikraken.api.public.ticker

This module queries the Ticker method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict
from decimal import Decimal

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import asset_pair_short
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv


def ticker(args):
    """Get currency ticker information."""

    # Parameters to pass to the API
    api_params = {
        'pair': args.pair,
    }

    res = query_api('public', 'Ticker', api_params, args)

    # the list will contain one OrderedDict containing
    # the parser ticker info per asset pair
    ticker_list = []

    for pair in res:
        # extract the results for the current pair
        pair_res = res[pair]

        # Initialize an OrderedDict to garantee the column order
        # for later use with the tabulate function
        pticker = OrderedDict()

        pticker['pair'] = asset_pair_short(pair)
        pticker['last'] = pair_res['c'][0]  # price only
        pticker['high'] = pair_res['h'][1]  # last 24h
        pticker['low'] = pair_res['l'][1]   # last 24h
        pticker['vol'] = pair_res['v'][1]   # last 24h
        pticker['wavg'] = pair_res['p'][1]  # last 24h

        # calculate an estimate of the traded volume in quoted currency
        # for the last 24h: Volume x Average price
        quote_val = Decimal(pticker['vol']) * Decimal(pticker['wavg'])

        unit_prefix = ''
        if quote_val >= 10e6:
            quote_val = quote_val / Decimal(1e6)
            unit_prefix = 'M'
        elif quote_val >= 10e3:
            quote_val = quote_val / Decimal(1e3)
            unit_prefix = 'k'

        pticker['vol value'] = str(round(quote_val)) + ' ' + unit_prefix + pticker['pair'][-3:]

        # get the price only
        pticker['ask'] = pair_res['a'][0]
        pticker['bid'] = pair_res['b'][0]

        ticker_list.append(pticker)

    if not ticker_list:
        return

    ticker_list = sorted(ticker_list, key=lambda pticker: pticker['pair'])

    if args.csv:
        print(csv(ticker_list, headers="keys"))
    else:
        print(tabulate(ticker_list, headers="keys"))



# -*- coding: utf8 -*-

"""
clikraken.api.private.get_deposit_methods

This module queries the DepositMethods method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import csv
from clikraken.clikraken_utils import _tabulate as tabulate


def get_deposit_methods(args=None):
    """Get deposit methods."""

    # Parameters to pass to the API
    api_params = {
        'asset': args.asset,
    }

    res = query_api('private', 'DepositMethods', api_params, args)

    m_list = []
    for method in res:
        # Initialize an OrderedDict to garantee the column order
        # for later use with the tabulate function
        method_dict = OrderedDict()
        # Remove leading Z or X from asset pair if it is of length 4
        method_dict['asset'] = args.asset[1:] if len(args.asset) == 4 and args.asset[0] in ['Z', 'X'] else args.asset
        method_dict['method'] = method['method']
        method_dict['fee'] = method['fee']
        method_dict['limit'] = method['limit']
        method_dict['gen-address'] = method['gen-address']
        m_list.append(method_dict)

    if not m_list:
        return

    # Sort alphabetically
    m_list = sorted(m_list, key=lambda method_dict: '{}{}'.format(method_dict['asset'], method_dict['method']))

    if args.csv:
        print(csv(m_list, headers="keys"))
    else:
        print(tabulate(m_list, headers="keys"))



# -*- coding: utf8 -*-

"""
clikraken.api.public.last_trades

This module queries the Trades method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import humanize_timestamp, base_quote_short_from_asset_pair
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv


def last_trades(args):
    """Get last trades."""

    _, quote_currency = base_quote_short_from_asset_pair(args.pair)

    # Parameters to pass to the API
    api_params = {
        'pair': args.pair,
    }
    if args.since:
        api_params['since'] = args.since

    res = query_api('public', 'Trades', api_params, args)

    results = res[args.pair]
    last_id = res['last']

    # initialize a list to store the parsed trades
    tlist = []

    # mappings
    ttype_label = {'b': 'buy', 's': 'sell'}
    otype_label = {'l': 'limit', 'm': 'market'}

    for trade in results:
        # Initialize an OrderedDict to garantee the column order
        # for later use with the tabulate function
        tdict = OrderedDict()
        tdict["Trade type"] = ttype_label.get(trade[3], 'unknown')
        tdict["Order type"] = otype_label.get(trade[4], 'unknown')
        tdict["Price"] = trade[0]
        tdict["Volume"] = trade[1]
        tdict["Age"] = humanize_timestamp(trade[2])
        # tdict["Misc"] = trade[5]
        tlist.append(tdict)

    if not tlist:
        return

    # Reverse trade list to have the most recent trades at the top
    tlist = tlist[::-1]

    if args.csv:
        print(csv(tlist[:args.count], headers="keys"))
    else:
        print(tabulate(tlist[:args.count], headers="keys") + '\n')

        # separate the trades based on their type
        sell_trades = [x for x in tlist if x["Trade type"] == "sell"]
        buy_trades = [x for x in tlist if x["Trade type"] == "buy"]

        last_sell = sell_trades[0]
        last_buy = buy_trades[0]
        lt = [
            ["", "Price (" + quote_currency + ")", "Volume", "Age"],
            ["Last Sell", last_sell["Price"], last_sell["Volume"], last_sell["Age"]],
            ["Last Buy", last_buy["Price"], last_buy["Volume"], last_buy["Age"]],
        ]

        print(tabulate(lt, headers="firstrow") + '\n')

        print('Last ID = {}'.format(last_id))



__all__ = [
    "__title__", "__summary__", "__url__", "__version__",
    "__author__", "__email__", "__license__",
]

__title__ = "clikraken"
__summary__ = "Command-line client for the Kraken exchange"
__url__ = "https://github.com/zertrin/clikraken"

__version__ = "0.8.3.2"

__author__ = "Marc Gallet"
__email__ = "zertrin@gmail.com"

__license__ = "Apache License, Version 2.0"



"""
Entrypoint module, in case you use `python -mnameless`.

Why does this file exist, and why __main__? For more info, read:
    - https://www.python.org/dev/peps/pep-0338/
    - https://docs.python.org/2/using/cmdline.html#cmdoption-m
    - https://docs.python.org/3/using/cmdline.html#cmdoption-m
"""

from clikraken.clikraken import main

if __name__ == "__main__":
    main()



# -*- coding: utf8 -*-

"""
clikraken.api.private.trades

This module queries the TradesHistory or QueryTrades method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import asset_pair_short
from clikraken.clikraken_utils import csv, format_timestamp
from clikraken.clikraken_utils import _tabulate as tabulate


def trades(args):
    """Get trades history or Query trades info"""

    # Parameters to pass to the API
    api_params = {
        # TODO: trades param
    }
    if args.type:
        api_params.update({'type': args.type})
    if args.start:
        api_params.update({'start': args.start})
    if args.end:
        api_params.update({'end': args.end})
    if args.ofs:
        api_params.update({'ofs': args.ofs})

    if args.id:
        api_params.update({
            'txid': args.id,
        })
        res_trades = query_api('private', 'QueryTrades', api_params, args)
    else:
        res = query_api('private', 'TradesHistory', api_params, args)
        # extract list of orders from API results
        res_trades = res['trades']

    # put all in a list
    tl = []
    for trade_id, trade_data in res_trades.items():
        trade_data.update({'tradeid': trade_id})
        tl += [trade_data]

    # filter orders based on currency pair
    if 'pair' in args and args.pair:
        tl = [td for td in tl if (td['pair'] in [args.pair, asset_pair_short(args.pair)] or args.pair == 'all')]

    # Lets get some order and filter some unnecessary fields
    tl2 = []
    for trade_data in tl:
        trade_dict = OrderedDict()
        trade_dict['txid'] = trade_data['tradeid']
        trade_dict['time'] = format_timestamp(int(trade_data['time']))
        trade_dict['pair'] = trade_data['pair'][1:] if len(trade_data['pair']) == 4 and \
            trade_data['pair'][0] in ['Z', 'X'] else trade_data['pair']
        trade_dict['type'] = trade_data['type']
        trade_dict['ordertype'] = trade_data['ordertype']
        trade_dict['vol'] = trade_data['vol']
        trade_dict['price'] = trade_data['price']
        trade_dict['cost'] = trade_data['cost']
        trade_dict['fee'] = trade_data['fee']
        trade_dict['margin'] = trade_data['margin']
        trade_dict['ordertxid'] = trade_data['ordertxid']
        trade_dict['misc'] = trade_data['misc']
        tl2.append(trade_dict)

    # sort orders by time
    tl2 = sorted(tl2, key=lambda x: x['time'])

    if not tl2:
        return

    if args.csv:
        print(csv(tl2, headers="keys"))
    else:
        print(tabulate(tl2, headers="keys"))



# -*- coding: utf8 -*-

"""
clikraken.api.private.list_open_positions

This module queries the OpenPositions method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from datetime import datetime
from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import _tabulate as tabulate


def list_open_positions(args):
    """List open positions."""

    # Parameters to pass to the API
    api_params = {
        'docalcs': 'true',
    }

    res = query_api('private', 'OpenPositions', api_params, args)

    pos_list = []
    for order in res.values():
        pos = OrderedDict()
        pos['ordertxid'] = order['ordertxid']
        pos['opening time'] = datetime.fromtimestamp(order['time'])
        pos['type'] = order['type']
        pos['volume'] = order['vol']
        pos['pair'] = order['pair']
        pos['ordertype'] = order['ordertype']
        pos['cost'] = order['cost']
        pos['fee'] = order['fee']
        pos['margin'] = order['margin']
        pos['value'] = order['value']
        pos['profit/loss'] = order['net']
        pos['rollover time'] = datetime.fromtimestamp(int(order['rollovertm']))
        pos['rollover terms'] = order['terms']

        pos_list.append(pos)

    print(tabulate(pos_list, headers="keys"))



# -*- coding: utf8 -*-

"""
clikraken.api.private.list_open_orders

This module queries the OpenOrders method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from decimal import Decimal

from clikraken.api.api_utils import parse_order_res, query_api
from clikraken.clikraken_utils import asset_pair_short
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv


def list_open_orders(args):
    """List open orders."""

    # Parameters to pass to the API
    api_params = {
        # TODO
    }
    if args.txid:
        api_params.update({'txid': args.txid})
        res_ol = query_api('private', 'QueryOrders', api_params, args)
    else:
        res = query_api('private', 'OpenOrders', api_params, args)
        # extract list of orders from API results
        res_ol = res['open']

    # the parsing is done in an helper function
    ol = parse_order_res(res_ol, ['open'])

    # filter and sort orders by price in each category
    for otype in ol:
        # filter orders based on currency pair
        if 'pair' in args and args.pair:
            ol[otype] = [odict for odict in ol[otype]
                         if (odict['pair'] in [args.pair, asset_pair_short(args.pair)] or args.pair == 'all')]
        # sort orders by price
        ol[otype] = sorted(ol[otype], key=lambda odict: Decimal(odict['price']))

    # final list is concatenation of buy orders followed by sell orders
    ol_all = ol['buy'] + ol['sell']

    if not ol_all:
        return

    if args.csv:
        print(csv(ol_all, headers="keys"))
    else:
        print(tabulate(ol_all, headers="keys"))



from clikraken.__about__ import (
    __title__, __summary__, __url__, __version__,
    __author__, __email__, __license__,
)

__all__ = [
    "__title__", "__summary__", "__url__", "__version__",
    "__author__", "__email__", "__license__",
]



# -*- coding: utf8 -*-

"""
clikraken.log_utils

This module sets the logger object up for use throughout clikraken's modules.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

import logging
import os
import sys

# fix issues with Unicode chars on Windows console
try:
    import win_unicode_console
except ImportError:
    pass
else:
    win_unicode_console.enable()

try:
    import colorlog
    have_colorlog = True
except ImportError:
    colorlog = None
    have_colorlog = False


class LessThanFilter(logging.Filter):
    """Custom logging filter letting LogRecord pass only if their level is below a limit"""

    def __init__(self, exclusive_maximum, name=""):
        super(LessThanFilter, self).__init__(name)
        self.max_level = exclusive_maximum

    def filter(self, record):
        # non-zero return means we log this message
        return record.levelno < self.max_level


def setup_logger():
    """Setup a colored logger if available and possible"""

    logger = logging.getLogger('clikraken')
    logger.setLevel(logging.DEBUG)

    # log to stdout from DEBUG to INFO
    ch_out = logging.StreamHandler(sys.stdout)
    ch_out.setLevel(logging.DEBUG)
    ch_out.addFilter(LessThanFilter(logging.WARNING))

    # WARNING and above goes to stderr
    ch_err = logging.StreamHandler(sys.stderr)
    ch_err.setLevel(logging.WARNING)

    format_str = '{levelname:8} - {message}'

    # basic formatter
    f = logging.Formatter(fmt=format_str, style='{')

    # colored formatted
    if have_colorlog:
        cf = colorlog.ColoredFormatter(fmt='{log_color}' + format_str, style='{')
    else:
        cf = f

    # only use the colored formatter when we are outputting to a terminal
    if os.isatty(2):
        ch_out.setFormatter(cf)
        ch_err.setFormatter(cf)
    else:
        ch_out.setFormatter(f)
        ch_err.setFormatter(f)

    logger.addHandler(ch_out)
    logger.addHandler(ch_err)

    return logger


logger = setup_logger()



# -*- coding: utf8 -*-

"""
clikraken.api.api_utils

This module contains various functions related to interacting
with or processing data from Kraken's API.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

import http.client
import socket
from collections import OrderedDict

import krakenex
import os

import clikraken.global_vars as gv
from clikraken.clikraken_utils import format_timestamp, print_results
from clikraken.log_utils import logger


def load_api_keyfile():
    """Load the Kraken API keyfile"""

    if not os.path.exists(gv.KRAKEN_API_KEYFILE):
        logger.warning("The API keyfile {} was not found!".format(gv.KRAKEN_API_KEYFILE))
        gv.API_KEY_LOADED = False

    # Instanciate the krakenex module to communicate with Kraken's API
    gv.KRAKEN_API = krakenex.API()

    if gv.API_KEY_LOADED is None:
        # Load the API key of the user
        gv.KRAKEN_API.load_key(gv.KRAKEN_API_KEYFILE)
        if gv.KRAKEN_API.key and gv.KRAKEN_API.secret:
            gv.API_KEY_LOADED = True
        else:
            gv.API_KEY_LOADED = False


def query_api(api_type, api_method, api_params, args):
    """
    Wrapper to query Kraken's API through krakenex
    and handle connection errors.
    """

    # Abort here if the API key isn't available and we are trying to query the private API
    if api_type == 'private' and not gv.API_KEY_LOADED:
        logger.critical('The API key must be set for private API queries! Aborting...')
        exit(2)

    # default to empty dict because that's the expected return type
    res = {}

    # just a mapping from api_type to the function to be called
    api_func = {
        'public': gv.KRAKEN_API.query_public,
        'private': gv.KRAKEN_API.query_private
    }
    # select the appropriate method depending on the api_type string
    func = api_func.get(api_type)

    # if cron mode is active, tone down some connection related errors in order to
    # not raise too many cron emails when Kraken is temporarily not available
    if gv.CRON:
        log = logger.info
    else:
        log = logger.error

    if func is not None:
        try:
            # call to the krakenex API
            res = func(api_method, api_params)
        except (socket.timeout, socket.error, http.client.BadStatusLine) as e:
            log('Network error while querying Kraken API!')
            log('Error details: ' + repr(e))
        except ValueError as e:
            log('Invalid response from Kraken API! '
                '(This can happen when Kraken API is overloaded. '
                'Try your luck again later.)')
            log('Error details: ' + repr(e))
        except Exception:
            logger.exception('Exception while querying Kraken API!')

    err = res.get('error', [])
    for e in err:
        # if cron mode is active, tone down some Kraken errors in order to not raise too many
        # cron emails when Kraken is temporarily not available, or for invalid nonces.
        if gv.CRON and e in ['EService:Unavailable', 'EAPI:Invalid nonce']:
            log = logger.info
        else:
            log = logger.error
        log('{}'.format(e))

    if args.raw:
        print_results(res)
        if not args.debug:
            exit(0)

    res = res.get('result')
    if not res:
        exit(0)

    return res


def parse_order_res(in_ol, status_list_filter=None):
    """
    Helper to parse the order results from the API.

    Depending on the status of the orders, different
    properties are available.

    See Kraken's API documentation for details.
    """

    # we will store the buy and sell orders separately during parsing
    ol = {'buy': [], 'sell': []}

    date_field = {
        'open': 'opentm',
        'closed': 'closetm',
        'canceled': 'closetm',
        'expired': 'closetm'
    }

    date_label = {
        'open': 'opening_date',
        'closed': 'closing_date',
        'canceled': 'closing_date',
        'expired': 'closing_date'
    }

    # status_list_filter is an optional argument of type list
    # the default value can't be set in the function signature
    if status_list_filter is None:
        status_list_filter = ['open', 'closed']

    # iterate over the given order list
    for txid in in_ol:
        # extract the current order dict
        o = in_ol[txid]

        ostatus = o['status']

        # We store the parsed orders in an OrderedDict to garantee
        # the order of columns for use later in the tabulate function.
        # (one key == "one column")
        odict = OrderedDict()

        odict['orderid'] = txid
        odict['status'] = ostatus
        odict['type'] = o['descr']['type']
        odict['vol'] = o['vol']

        # Volume executed is only available if the order isn't open
        # TODO: check if that's really true
        if ostatus != 'open':
            odict['vol_exec'] = o['vol_exec']

        odict['pair'] = o['descr']['pair']
        odict['ordertype'] = o['descr']['ordertype']

        # If the order is open, take the price from the order description
        # if the order is closed, take the average price
        if ostatus == 'open':
            odict['price'] = o['descr']['price']
        else:
            odict['price'] = o['price']

        # Cost and Fee are only available if the order isn't open
        if ostatus != 'open':
            odict['cost'] = o['cost']
            odict['fee'] = o['fee']

        odict['viqc'] = ('viqc' in o['oflags'])  # boolean check

        # date_label[ostatus] is the column name and changes
        # depending on the status of the order.
        odict[date_label[ostatus]] = format_timestamp(o[date_field[ostatus]])

        # filter here based on the list of status that we want
        if ostatus in status_list_filter:
            ol.get(odict['type']).append(odict)

    return ol



# -*- coding: utf8 -*-

"""
clikraken.api.private.cancel_order

This module queries the CancelOrder method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from clikraken.api.api_utils import query_api
from clikraken.log_utils import logger


def cancel_order(args):
    """Cancel an open order."""

    for order_id in args.order_ids:
        # Parameters to pass to the API
        api_params = {
            'txid': order_id,
        }

        res = query_api('private', 'CancelOrder', api_params, args)

        count = res.get('count')
        pending = res.get('pending')

        if count:
            print('{} - count: {}'.format(order_id, count))
        if pending:
            logger.info('{} - order(s) is/are pending cancellation!'.format(order_id))



# -*- coding: utf8 -*-

"""
clikraken.clikraken_utils

This module contains various functions that are used throughout
clikraken's modules.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

import arrow
import configparser
import json
import os
from functools import partial

from tabulate import tabulate

import clikraken.global_vars as gv
from clikraken import __version__
from clikraken.log_utils import logger


_tabulate = partial(tabulate, floatfmt='.12g')


def load_config():
    """Load configuration parameters from the settings file"""

    if not os.path.exists(gv.USER_SETTINGS_PATH):
        logger.info("The user settings file {} was not found! "
                    "Using hardcoded default values.".format(gv.USER_SETTINGS_PATH))

    config = configparser.ConfigParser()
    config.read_string(gv.DEFAULT_SETTINGS_INI)
    config.read(gv.USER_SETTINGS_PATH)

    conf = config['clikraken']

    # Get the default currency pair from environment variable if available
    # otherwise take the value from the config file.
    gv.DEFAULT_PAIR = os.getenv('CLIKRAKEN_DEFAULT_PAIR', conf.get('currency_pair'))
    gv.TICKER_PAIRS = os.getenv('CLIKRAKEN_TICKER_PAIRS', conf.get('ticker_currency_pairs'))

    # Get the default asset pair from the config file
    gv.DEFAULT_ASSET = os.getenv('CLIKRAKEN_DEFAULT_ASSET', conf.get('asset'))

    gv.TZ = conf.get('timezone')
    gv.TRADING_AGREEMENT = conf.get('trading_agreement')


def version(args=None):
    """Print program version."""
    print('clikraken version: {}'.format(__version__))


def humanize_timestamp(ts):
    """Humanize a UNIX timestamp."""
    return arrow.get(ts).humanize()


def format_timestamp(ts):
    """Format a UNIX timestamp to truncated ISO8601 format."""
    return arrow.get(ts).to(gv.TZ).replace(microsecond=0).format('YYYY-MM-DD HH:mm:ssZZ')


def print_results(res):
    """Pretty-print the JSON result from the API."""
    if res is not None:
        print(json.dumps(res, indent=2))


def base_quote_short_from_asset_pair(ap_str):
    """Try to identify the short version of the base and quote of the asset pair"""

    ap_str = ap_str.upper()

    if len(ap_str) == 8:
        # XABCZDEF
        base = ap_str[1:4] if ap_str[0] in ['Z', 'X'] else ap_str[:4]
        quote = ap_str[5:] if ap_str[4] in ['Z', 'X'] else ap_str[4:]
    elif len(ap_str) == 6:
        # ABCDEF
        base = ap_str[:3]
        quote = ap_str[3:]
    elif len(ap_str) == 7:
        if ap_str[-4] in ['Z', 'X']:
            # ABCXDEF
            base = ap_str[:3]
            quote = ap_str[4:]
        elif ap_str[0] in ['Z', 'X']:
            # XABCDEF
            base = ap_str[1:4]
            quote = ap_str[4:]
        else:
            # ABCDEFG
            # assume EFG is the quote ¯\_(ツ)_/¯
            base = ap_str[:4]
            quote = ap_str[4:]

    return base, quote


def asset_pair_short(ap_str):
    """Convert XETHZEUR to ETHEUR"""
    base, quote = base_quote_short_from_asset_pair(ap_str)
    return base + quote


def check_trading_agreement():
    if gv.TRADING_AGREEMENT != 'agree':
        logger.warn('Before being able to use the Kraken API for market orders, '
                    'orders that trigger market orders, trailing stop limit orders, and margin orders, '
                    'you need to agree to the trading agreement at https://www.kraken.com/u/settings/api '
                    'and set the parameter "trading_agreement" to "agree" in the settings file '
                    '(located at ' + gv.USER_SETTINGS_PATH + '). If the settings file does not yet exists, '
                    'you can generate one by following the instructions in the README.md file.')


def output_default_settings_ini(args):
    """Output the contents of the default settings.ini file"""
    print(gv.DEFAULT_SETTINGS_INI)


def csv(items, headers=None):
    output = []
    # Headers
    if headers is not None:
        if headers == "keys":
            headers = items[0].keys()
        output += [headers]
    # Items
    for item in items:
        it = [val for val in item.values()]
        output += [it]
    # Render output as txt
    return '\n'.join([(gv.CSV_SEPARATOR).join([str(i) for i in o]) for o in output])



# -*- coding: utf8 -*-

"""
clikraken.api.private.get_ledgers

This module queries the Ledgers or QueryLedgers method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv
from clikraken.clikraken_utils import format_timestamp


def get_ledgers(args):
    """Get ledgers info"""

    # If id is specified, then query just that
    if args.id:
        api_params = {
            'id': args.id,
        }
        lg = query_api('private', 'QueryLedgers', api_params, args)
    else:
        # Parameters to pass to the API
        api_params = {
        }
        if args.asset:  # We do not user DEFAULT_ASSET HERE as this param is optional
            api_params.update({'asset': args.asset})
        if args.type:
            api_params.update({'type': args.type})
        if args.start:
            api_params.update({'start': args.start})
        if args.end:
            api_params.update({'end': args.end})
        if args.ofs:
            api_params.update({'ofs': args.ofs})
        res = query_api('private', 'Ledgers', api_params, args)
        # extract list of ledgers from API results
        lg = res['ledger']

    lg_list = []
    for refid, item in lg.items():
        # Initialize an OrderedDict to garantee the column order
        # for later use with the tabulate function
        asset_dict = OrderedDict()
        asset_dict['id'] = refid
        asset_dict['refid'] = item['refid']
        asset_dict['time'] = format_timestamp(int(item['time']))
        asset_dict['type'] = item['type']
        # Remove leading Z or X from item pair if it is of length 4
        asset_dict['asset'] = item['asset'][1:] \
            if len(item['asset']) == 4 and item['asset'][0] in ['Z', 'X'] else item['asset']
        asset_dict['aclass'] = item['aclass']
        asset_dict['amount'] = float(item['amount'])
        asset_dict['fee'] = float(item['fee'])
        asset_dict['balance'] = float(item['balance'])

        lg_list.append(asset_dict)

    if not lg_list:
        return

    # sort by date
    lg_list = sorted(lg_list, key=lambda odict: odict['time'])

    if args.csv:
        print(csv(lg_list, headers="keys"))
    else:
        print(tabulate(lg_list, headers="keys"))



# -*- coding: utf8 -*-

"""
clikraken.api.public.asset_pairs

This module queries the AssetPairs method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv


def asset_pairs(args):
    """Get available asset pairs."""

    # Parameters to pass to the API
    api_params = {}

    res = query_api('public', 'AssetPairs', api_params, args)

    # initialize a list to store the parsed assets pairs
    assetlist = []

    for assetpair in res:
        if assetpair.endswith('.d'):
            continue
        ad = OrderedDict()
        ad['Pair'] = assetpair
        ad['Alt Name'] = res[assetpair]['altname']
        ad['Base'] = res[assetpair]['base']
        ad['Quote'] = res[assetpair]['quote']
        assetlist.append(ad)

    if args.csv:
        print(csv(assetlist, headers="keys"))
    else:
        print(tabulate(assetlist, headers='keys'))
        print('--- Total: {} pairs'.format(len(assetlist)))



# -*- coding: utf8 -*-

"""
clikraken.api.private.get_deposit_addresses

This module queries the DepositAddresses method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import csv, format_timestamp
from clikraken.clikraken_utils import _tabulate as tabulate


def get_deposit_addresses(args=None):
    """Get deposit addresses."""

    # Parameters to pass to the API
    api_params = {
        'asset': args.asset,
        'method': args.method,
    }
    if args.new:
        api_params['new'] = args.new

    res = query_api('private', 'DepositAddresses', api_params, args)

    if not res:
        return

    if args.one:
        # Get preferably not used addresses
        addresses = [a for a in res if a.get('new', False)] or res
        print(addresses[0]['address'])
        return

    addresses_list = []
    for address in res:
        # Initialize an OrderedDict to garantee the column order
        # for later use with the tabulate function
        asset_dict = OrderedDict()
        # Remove leading Z or X from asset pair if it is of length 4
        asset_dict['asset'] = args.asset[1:] if len(args.asset) == 4 and args.asset[0] in ['Z', 'X'] else args.asset
        asset_dict['address'] = address['address']
        asset_dict['new'] = address.get('new', False)
        asset_dict['expiretm'] = \
            int(address.get('expiretm', 0)) > 0 and format_timestamp(int(address.get('expiretm', 0))) or ''
        addresses_list.append(asset_dict)

    if not addresses_list:
        return

    # sort by expiretm
    addresses_list = sorted(addresses_list, key=lambda odict: odict['expiretm'])

    if args.csv:
        print(csv(addresses_list, headers="keys"))
    else:
        print(tabulate(addresses_list, headers="keys"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

# I really prefer Markdown to reStructuredText. PyPi does not. This allows me
# to have things how I'd like, but not throw complaints when people are trying
# to install the package and they don't have pypandoc or the README in the
# right place.
try:
    import pypandoc
    long_description = pypandoc.convert_file('README.md', 'rst')
except (IOError, ImportError):
    long_description = open('README.md').read()

about = {}
with open('src/clikraken/__about__.py') as f:
    exec(f.read(), about)
# now we have a about['__version__'] variable

setup(
    name=about['__title__'],
    version=about['__version__'],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    author=about['__author__'],
    author_email=about['__email__'],
    license=about['__license__'],
    description=about['__summary__'],
    long_description=long_description,
    include_package_data=True,
    url=about['__url__'],
    install_requires=[
        'krakenex>=0.1,<1.0',
        'arrow<0.14',
        'tabulate',
        'colorlog',
    ],
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Office/Business :: Financial",
        "Topic :: Office/Business :: Financial :: Investment",
    ],
    entry_points={
        'console_scripts': [
            'clikraken=clikraken.clikraken:main',
        ],
    },
)



# -*- coding: utf8 -*-

"""
clikraken.api.private.place_order

This module queries the AddOrder method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

import clikraken.global_vars as gv

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import check_trading_agreement
from clikraken.log_utils import logger


def place_order(args):
    """Place an order."""

    # Parameters to pass to the API
    api_params = {
        'pair': args.pair,
        'type': args.type,
        'ordertype': args.ordertype,
        'volume': args.volume,
        'starttm': args.starttm,
        'expiretm': args.expiretm,
        'leverage': args.leverage,
    }

    if gv.TRADING_AGREEMENT == 'agree':
        api_params['trading_agreement'] = 'agree'

    if args.ordertype == 'limit':
        if args.price is None:
            logger.error('For limit orders, the price must be given!')
            return
        else:
            api_params['price'] = args.price
    elif args.ordertype == 'market':
        if args.price is not None:
            logger.warn('price is ignored for market orders!')
        check_trading_agreement()

    if args.userref:
        api_params['userref'] = args.userref

    oflags = []  # order flags
    if args.ordertype == 'limit':
        if not args.nopost:
            # for limit orders, by default set post-only order flag
            oflags.append('post')
    if args.viqc:
        oflags.append('viqc')
    if oflags:
        api_params['oflags'] = ','.join(oflags)

    if args.validate:
        api_params['validate'] = 'true'

    res = query_api('private', 'AddOrder', api_params, args)

    descr = res.get('descr')
    odesc = descr.get('order', 'No description available!')
    print(odesc)

    txid = res.get('txid')

    if not txid:
        if args.validate:
            logger.info('Validating inputs only. Order not submitted!')
        else:
            logger.warn('Order was NOT successfully added!')
    else:
        for tx in txid:
            print(tx)



# -*- coding: utf8 -*-

"""
clikraken.api.private.get_balance

This module queries the Balance method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv


def get_balance(args=None):
    """Get user balance."""

    # Parameters to pass to the API
    api_params = {}

    res = query_api('private', 'Balance', api_params, args)

    bal_list = []
    for asset in res:
        # Initialize an OrderedDict to garantee the column order
        # for later use with the tabulate function
        asset_dict = OrderedDict()
        # Remove leading Z or X from asset pair if it is of length 4
        asset_dict['asset'] = asset[1:] if len(asset) == 4 and asset[0] in ['Z', 'X'] else asset
        asset_dict['balance'] = res[asset]
        bal_list.append(asset_dict)

    if not bal_list:
        return

    # Sort alphabetically
    bal_list = sorted(bal_list, key=lambda asset_dict: asset_dict['asset'])

    if args.csv:
        print(csv(bal_list, headers="keys"))
    else:
        print(tabulate(bal_list, headers="keys"))



# -*- coding: utf8 -*-

"""
clikraken.api.public.depth

This module queries the Depth method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict
from decimal import Decimal

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import asset_pair_short, humanize_timestamp
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv


def depth(args):
    """Get market depth information."""

    # Parameters to pass to the API
    api_params = {
        'pair': args.pair,
        'count': args.count
    }

    res = query_api('public', 'Depth', api_params, args)

    depth_dict = {'asks': [], 'bids': []}
    depth_label = {'asks': "Ask", 'bids': "Bid"}

    shortpair = asset_pair_short(args.pair)

    # dtype is 'asks' or 'bids'
    for dtype in depth_dict:
        # extract the array of market depth from the api results
        dlist = res[args.pair][dtype]
        # build a column label depending on the asset pair and dtype
        price_label = shortpair + " " + depth_label[dtype]

        for delem in dlist:
            # Initialize an OrderedDict to garantee the column order
            # for later use with the tabulate function
            dentry = OrderedDict()
            dentry[price_label] = delem[0]
            dentry["Volume"] = delem[1]
            dentry["Age"] = humanize_timestamp(delem[2])
            depth_dict[dtype].append(dentry)

        if not dlist:
            continue

        # sort by price descending
        depth_dict[dtype] = reversed(sorted(depth_dict[dtype], key=lambda dentry: Decimal(dentry[price_label])))

    if args.csv:
        output = []
        for dtype in depth_dict.keys():
            for o in depth_dict[dtype]:
                it = OrderedDict()
                it['dtype'] = dtype
                for k, v in o.items():
                    if len(k.split(' ')) > 1:
                        # key has a space, this is the "price_label" column -> "XABCZDEF Ask"
                        it['pair'] = k.split(' ')[0]  # keep only "XABCZDEF"
                        it['price'] = v
                    else:
                        # the other columns don't contain a space
                        it[k] = v
                output += [it]
        print(csv(output, headers="keys"))
    else:
        asks_table = tabulate(depth_dict['asks'], headers="keys")
        bids_table = tabulate(depth_dict['bids'], headers="keys")
        print("{}\n\n{}".format(asks_table, bids_table))



# -*- coding: utf8 -*-

"""
clikraken.api.public.ohlc

This module queries the OHLC method of Kraken's API
and outputs the results in a tabular format.

Licensed under the Apache License, Version 2.0. See the LICENSE file.
"""

from collections import OrderedDict

from clikraken.api.api_utils import query_api
from clikraken.clikraken_utils import format_timestamp, asset_pair_short
from clikraken.clikraken_utils import _tabulate as tabulate
from clikraken.clikraken_utils import csv


def ohlc(args):
    """Get OHLC data for asset pairs for various minute intervals:
    1 (default), 5, 15, 30, 60, 240, 1440, 10800, 21600."""

    # Parameters to pass to the API
    api_params = {
        'pair': args.pair,
    }
    if args.since:
        api_params['since'] = args.since

    if args.interval:
        api_params['interval'] = interval = args.interval
    else:
        interval = 1

    res = query_api('public', 'OHLC', api_params, args)

    results = res[args.pair]
    last_id = res['last']

    # initialize a list to store the parsed ohlc data
    ohlclist = []

    for period in results:
        # Initialize an OrderedDict to garantee the column order
        # for later use with the tabulate function
        ohlcdict = OrderedDict()
        ohlcdict["Time"] = format_timestamp(period[0])
        ohlcdict["Open"] = period[1]
        ohlcdict["High"] = period[2]
        ohlcdict["Low"] = period[3]
        ohlcdict["Close"] = period[4]
        ohlcdict["VWAP"] = period[5]
        ohlcdict["Volume"] = period[6]
        ohlcdict["Count"] = period[7]
        ohlclist.append(ohlcdict)

    if not ohlclist:
        return

    # Reverse trade list to have the most recent interval at the top
    ohlclist = ohlclist[::-1]

    if args.csv:
        print(csv(ohlclist[:args.count], headers="keys"))
    else:
        print('Asset pair: ' + asset_pair_short(args.pair))
        print('Interval: ' + str(interval) + 'm\n')

        print(tabulate(ohlclist[:args.count], headers="keys") + '\n')

        print('Last ID = {}'.format(last_id))
